if (!window.location.origin) {
    window.location.origin = window.location.protocol + "//" + window.location.hostname + (window.location.port ? ':' + window.location.port : '');
}

function defer(method) {
    if (document.readyState === "complete" && window.jQuery) {
        (function ($) {
            $.event.special.destroyed = {
                remove: function (o) {
                    if (o.handler) {
                        o.handler()
                    }
                }
            }
        })(jQuery)
        method();
    }
    else
        setTimeout(function () { defer(method) }, 50);
}

/* ****************************************************************************** */
/* ****************************************************************************** */
/* This section contains code that prepares the search category trees             */
/* ****************************************************************************** */
/* ****************************************************************************** */

function LoadCategoryData() {
    if (window.categoryData === null || typeof window.categoryData === "undefined") {
        $.getJSON('/category-data/',
            function (data) {
                window.categoryData = SortObjects(data, "title");
                PopulateCategoryData();
            });
    }
    else PopulateCategoryData();      
}

function SetInitialSearchFilter() {
    if (defaultSearchToDashboard && window.currentDashboard_id != null) {
        var option = $('option[value=' + window.currentDashboard_id + ']');
        if (option.length < 1) return;
        var label = $('option[value=' + window.currentDashboard_id + ']').first().text();
        UpdateMainSearchSelection(window.currentDashboard_id, label);
    }
}

function PopulateCategoryData() {
    ConfigurePage(GetQuickSearchULObject("quick"), GetQuickSearchSelectObject("quick"), "quick");

    ConfigureHomeOptions();

    if (GetMainSearchULObject("full") !== 0)
        ConfigurePage(GetMainSearchULObject("full"), GetMainSearchSelectObject("full"), "full");

    $('#search-filterfull, #search-filterquick').click(CategoryMenuClick);
    var categorySelected = $(".navbar select[name='category']");
    UpdateMainSearchSelection(categorySelected.val(), categorySelected.find("option:selected").text());

    SetInitialSearchFilter();
}

function SortObjects(array, property) {
    return array.sort(function (a, b) {
        if (a[property] < b[property]) return -1;
        if (a[property] > b[property]) return 1;
        return 0;
    });
}

function ConfigureHomeOptions() {
    var homeButton = $(".dropdown-toggle[title='Home']");
    homeButton.click(ProcessClickToHome);

    var topCategory = GetTopCategoryRecord();
    var homeList = GetHomeULObject();
    var homeCategories = GetHomeCategories();

    // the main search window
    var firstListCount = $(homeList).children('li').length;
    if (typeof firstListCount === "undefined" || firstListCount === 0) {
        setTimeout(ConfigureHomeOptions, 100);
        return;
    }

    StyleCategoryDropdown(homeList, "home");
    ClearContents(homeList);
    if (defaultDashboard_id != window.currentDashboard_id && !(defaultDashboard_id == "" && window.currentDashboard_id == null))
        AddHomeDefaultOption(homeList);
    AddHomeItem(homeList, GetHomeFilterObject());
    var children = GetChildCategories(topCategory, homeCategories);

    for (var i = 0; i < children.length; i++) {
        WalkTheTree(children[i], AddHomeItem, homeList, homeCategories, false, false, true, null);
    }

	var sHtml = '<iframe id="fake" src="about:blank" style="border: currentColor; border-image: none; left: 0px; top: 0px; width: 100%; height: 100%; display: inline; position: absolute; z-index: -1;"></iframe>';
	$(sHtml).appendTo(homeList);

    $("#" + defaultDashboard_id + "-dashboardSelected").show();
    GenerateTree("home", true);
}

function ProcessClickToHome() {
    var homeSelection = "Home";

    var mf = getParameterByName("mf");
    if (mf != null) {
        var pr = getParameterByName("pr", ("?" + mf));
        if (pr != null)
            homeSelection = pr.replace(/\%/g, "");
    }

    ResetTreeToGuid(homeSelection, "home", true);
    SetFocus("home", homeSelection);
}

function ConfigurePage(firstMenu, firstSelect, identifier) {
    var topCategory = GetTopCategoryRecord();
    var searchCategories = GetSearchCategories();

    // the main search window
    var firstListCount = $(firstMenu).children('li').length;
    if (typeof firstListCount === "undefined" || firstListCount === 0)
    {
        setTimeout(ConfigurePage, 100);
        return;
    }

    StyleCategoryDropdown(firstMenu, identifier);

    ClearContents(firstMenu);
    AddListItem(firstMenu, GetAllSearchObject());
    var children = GetChildCategories(topCategory, searchCategories);

    for (var i = 0; i < children.length; i++) {
        WalkTheTree(children[i], AddListItem, firstMenu, searchCategories, false, false, true, null);
    }

    ClearContents(firstSelect);
    AddSelectItem(firstSelect, GetAllSelectObject());
    WalkTheTree(topCategory, AddSelectItem, firstSelect, searchCategories, false, false, false, null);

    GenerateTree(identifier, true);

    var categoryGuid = getParameterByName("category");
    if (categoryGuid === null || categoryGuid === "All" || categoryGuid === "")
    {
        $('input[type="text"]#filter' + identifier).val("");
        $('button#search-filter' + identifier + ' span.selected').text("All");
    }
    else
    {
        $('input[type="text"]#filter' + identifier).val(categoryGuid);
        UnFoldTreeToGuid(categoryGuid, identifier);
        var label = $('a[data-value="' + categoryGuid + '"]').attr('data-label');
        $('button#search-filter' + identifier + ' span.selected').text(label);
    }
    if (categoryGuid !== null && typeof categoryGuid !== "undefined" && categoryGuid !== "")
        $('option[value=' + categoryGuid + ']').attr("selected", "selected");
}

function StyleCategoryDropdown(object, identifier) {
    $(object).addClass("tree");
    $(object).data("value", identifier);
    $(object).css("max-height", "400px");
    $(object).css("min-width", "220px");
    $(object).css("overflow-y", "auto");
    $(object).css("white-space", "nowrap");
}

function GenerateULElement(node) {
    return $('<ul></ul>').appendTo(node);
}

function WalkTheTree(node, functionToRun, nodeForFunction, categories, entry, hide, isAddListItem, levels) {
    if (node === null || typeof node === "undefined")
        return;

    var liNode = nodeForFunction;
    isAddListItem = (isAddListItem !== null && typeof isAddListItem !== "undefined") ? isAddListItem : false;

    if (entry && isAddListItem)
    {
        nodeForFunction = GenerateULElement(nodeForFunction);
    }
    
    if (isAddListItem)
        liNode = functionToRun(nodeForFunction, node.title, node.id);
    else
        functionToRun(nodeForFunction, node.title, node.id);

    if (hide !== null && hide !== false)
        $(liNode).css('display', 'none');

    if (levels != null && levels < 1) return nodeForFunction;
    if (levels != null) levels--;

    var children = GetChildCategories(node, categories);
    if (children.length === 0)
    {
        var anchor = GetChildAnchor(liNode);
        $(anchor).css('margin-left', '1.5em');
    }
    else
    {
        for (var i = 0; i < children.length; i++)
        {
            if (i === 0)
                var newNodeForFunction = WalkTheTree(children[i], functionToRun, liNode, categories, true, false, isAddListItem, levels);
            else
                WalkTheTree(children[i], functionToRun, newNodeForFunction, categories, false, false, isAddListItem, levels);
        }
    }

    return nodeForFunction;
}

function AddJQueryBindFirstMethod() {
    $.fn.bindFirst = function(name, fn) {
        var elem, handlers, i, _len;
        this.bind(name, fn);
        for (i = 0, _len = this.length; i < _len; i++) {
            elem = this[i];
            handlers = jQuery._data(elem).events[name.split('.')[0]];
            handlers.unshift(handlers.pop());
        }
    };
}

function UpdateMainSearchSelectionEvent(e) {
    var guid = e.currentTarget.dataset.value;
    var label = e.currentTarget.dataset.label;
    UpdateMainSearchSelection(guid, label);
    $(e.currentTarget).closest(".input-group").find("input[name='q']").focus();
}

function UpdateMainSearchSelection(guid, label) {
    //$('option[value=' + guid + ']').attr("selected", "selected");

    //Above line of code is not searching the articles and fixing this with below line
    $("form select[name='category']").val(guid).change();

    $('button#search-filterquick span.selected').text(label).attr;
    $('button#search-filterfull span.selected').text(label).attr;

    ResetTreeToGuid(guid, "quick", true);
    ResetTreeToGuid(guid, "full", true);
}

function ResetTreeToGuid(guid, identifier, foldFirst) {
    FoldEntireTree(identifier);
    if (!foldFirst) {
        UnfoldFirstLevel(identifier);
    }
    UnFoldTreeToGuid(guid, identifier);
}

function ClearContents(menuObject) {
    $(menuObject).empty();
}

function AddListItem(menuObject, name, value) {

    if (value === null || typeof value === "undefined")
    {
        return $(name).appendTo(menuObject);
    }

    value = value.replace('-','');
    value = value.replace('-','');
    value = value.replace('-','');
    value = value.replace('-','');

    var re = new RegExp(/(.*) (?:\(.*\))$/g);
    var result = re.exec(name);
    if (result !== null)
        name = result[1];
    re.lastIndex = 0;

    var sHtml = 
        '<li role="presentation"> \
            <a href="#" role="option" data-value="svalue" data-label="sname" aria-label="svalue" aria-selected="false" tabindex="-1" style="display: inline-block" onclick="UpdateMainSearchSelectionEvent(event)">sname</a> \
        </li>';
    sHtml = sHtml.replace('sname', name);
    sHtml = sHtml.replace('sname', name);
    sHtml = sHtml.replace('svalue', value);
    sHtml = sHtml.replace('svalue', value);

    return $(sHtml).appendTo(menuObject);
}

function AddHomeDefaultOption(menuObject) {
    var sHtml =
        GetSetDefaultDashboardObject() +
        '<li id="setDefaultDashboard_divider" class="divider"></li>';
    $(sHtml).appendTo(menuObject);
}

function AddHomeItem(menuObject, name, id) {

    if (id === null || typeof id === "undefined") {
        return $(name).appendTo(menuObject);
    }

    id = id.replace(/-/g, '');

    var re = new RegExp(/(.*) (?:\(.*\))$/g);
    var result = re.exec(name);
    if (result !== null)
        name = result[1];
    re.lastIndex = 0;

    var sHtml =
        '<li role="presentation"> \
            <a href="surl" role="option" data-value="svalue" data-label="sname" aria-label="svalue" aria-selected="false" tabindex="-1" style="display: inline-block">sname</a> \
            <span id="svalue-dashboardSelected" class="glyphicon glyphicon-ok dashboard-selected"></span> \
        </li>';
    sHtml = sHtml.replace(/sname/g, name);
    sHtml = sHtml.replace(/svalue/g, id);
    sHtml = sHtml.replace('surl', GetHomeItemUrl(name, id));

    return $(sHtml).appendTo(menuObject);
}

function AddSelectItem(menuObject, name, value) {

    if (value === null || typeof value === "undefined")
    {
        return $(name).appendTo(menuObject);
    }

    value = value.replace('-','');
    value = value.replace('-','');
    value = value.replace('-','');
    value = value.replace('-','');

    var sHtml = '<option value="svalue">sname</option>';
    sHtml = sHtml.replace('sname', name);
    sHtml = sHtml.replace('svalue', value);

    return $(sHtml).appendTo(menuObject);
}

function GetQuickSearchULObject() {
    var r = $('ul.weblinks').find('ul.dropdown-menu.dropdown-search').find('li.dropdown-submenu.dropdown').find('ul.dropdown-menu');
    if (r.length === 0)
        return $('ul.weblinks').find('ul.dropdown-menu.tree').not(".dropdown-search");
    return r;
}
function GetMainSearchULObject(identifier) {
    var r = $('section.section-landing, section.section-landing-search').find('ul.dropdown-menu').not(".dropdown-search");
    if (r.length === 0)
        return $('#logicalNames' + identifier).next().find('ul.dropdown-menu');
    return r;
}
function GetHomeULObject() {
    var r = $('ul.weblinks').find('li a.dropdown-toggle[title="Home"]').siblings('.dropdown-menu');
    return r;
}
function GetQuickSearchSelectObject(identifier) {
    var r = $('ul.weblinks').find('ul.dropdown-menu.dropdown-search').find('select[name="category"]');
    if (r.length === 0)
        return $('#logicalNames' + identifier).next().find('select[name="category"]');
    return r;
}
function GetMainSearchSelectObject(identifier) {
    var r = $('section.section-landing, section.section-landing-search').find('select[name="category"]');
    if (r.length === 0)
        return $('#logicalNames' + identifier).next().find('select[name="category"]');
    return r;
}
function GetQuickSearchInputObject() {
    var r = $('ul.weblinks').find('ul.dropdown-menu.dropdown-search').find("input[name='q']");
    return r;
}

function GetULSearchObjects() {
    return $('ul.dropdown-menu').not(".dropdown-search");
}

function GetAllSearchObject() {
    return '<li role="presentation"> \
            <a href="#" role="option" data-value="All" data-label="All" aria-label="All" aria-selected="false" tabindex="-1" style="display: inline-block; margin-left: 22px; margin-bottom: 4px;" onclick="UpdateMainSearchSelectionEvent(event)">All</a> \
            </li>';
}

function GetHomeFilterObject() {
    return '<li role="presentation"> \
            <a href="/home/?home" role="option" data-value="Home" data-label="Home" aria-label="Home" aria-selected="false" tabindex="-1" style="display: inline-block; margin-left: 22px; margin-bottom: 4px;">Home</a> \
            </li>';
}

function GetSetDefaultDashboardObject() {
    return '<li id="setDefaultDashboard_li" role="presentation"> \
            <a href="#" tabindex="-1" onclick="SetDefaultDashboard()" style="display: inline-block; margin-left: 22px; margin-bottom: 4px;">Set Default Homepage</a> \
            </li>';
}

function GetAllSelectObject() {
    return '<option value="All" selected="selected">All</option>';
}

function GetTopCategoryRecord() {
    for (var i = 0; i < window.categoryData.length; i++)
    {
        var currentCategory = window.categoryData[i];
        if (!currentCategory.parentid)
            return currentCategory;
    }
}

function GetHomeCategories() {
    var programs = [];

    for (var i = 0; i < window.categoryData.length; i++) {
        if (window.categoryData[i].program.toLowerCase() == 'false')
            continue;

        var curEntity = window.categoryData[i];
        while (curEntity != undefined && curEntity.parentid) {
            if (programs.indexOf(curEntity) < 0)
                programs.push(curEntity);
            curEntity = $.grep(window.categoryData, function (e) { return e.id == curEntity.parentid })[0];
        }
    }

    return programs;
}

function GetSearchCategories() {
    var searchCat = [];

    for (var i = 0; i < window.categoryData.length; i++) {
        if (window.categoryData[i].id.replace(/-/g, "").toUpperCase() == ProductCategoryId)
            continue;
        searchCat.push(window.categoryData[i]);
    }

    return searchCat;
}

function GetChildCategories(parentCategory, categories) {
    var children = [];

    if (parentCategory === null 
        || typeof parentCategory === "undefined"
        || typeof parentCategory.id === "undefined")
        return [];

    for (var i = 0; i < categories.length; i++) {
        var currentCategory = categories[i];

        if (typeof currentCategory.parentid !== "string")
            continue;

        if (currentCategory.parentid.toLowerCase() === parentCategory.id.toLowerCase())
            children[children.length] = currentCategory;
    }

    return children;
}

/* ****************************************************************************** */
/* ****************************************************************************** */
/* This section contains code that generates a category tree for filtering search */
/* ****************************************************************************** */
/* ****************************************************************************** */
function GenerateTree(identifier, foldFirst) {
    var treeNodes = GetTreeNodes(identifier);
    $(treeNodes).css('padding-left', '0px');

    // adding onclick directly to label to ensure event triggers first
    $('<label onclick="ToggleTreeLimb(event);"><input type="checkbox"><span>+</span></label>').prependTo(treeNodes);
    
    FoldEntireTree(identifier);
    if (!foldFirst) {
        UnfoldFirstLevel(identifier);
    }
}

function CategoryMenuClick(e) {
    var identifier = event.currentTarget.id.replace("search-filter", "");
    SetFocus(identifier, $('input[type="text"]#filterquick').val());
}

function SetFocus(identifier, dataValue) {
    setTimeout(function (identifier) {
        var list = $('ul.tree').filter(function () { return $(this).data("value") === identifier; });
        if (list.is(':visible')) {
            if (!dataValue) dataValue = "All";
            list.find("[data-value='" + dataValue + "']").focus();
        }
    }, 1, identifier);
}

function ToggleTreeLimb(e) { 
    e.stopPropagation();
    var liNode = GetParentLiNodeFromChildElement(e.target);
    var checkboxInput = $(liNode).children('label').children('input[type="checkbox"]');
    if (IsChecked(checkboxInput))
        UnfoldNode(liNode);
    else
        FoldNode(liNode);
}

function GetParentLiNodeFromChildElement(target) {
    if ($(target).parent().is('li'))
        return $(target).parent();
    else if ($(target).parent().parent().is('li'))
        return $(target).parent().parent();
    else
        return null;
}

function GetTreeNodes(identifier) {
    var node = $('.tree').filter(function() { return $(this).data("value") === identifier; });
    return $(node).find('li').has('ul');
}

function IsChecked(checkboxNode) {
    return $(checkboxNode).prop('checked');
}

function UnfoldNode(listNode) {
    var child = GetChildSpan(listNode);
    var input = GetChildInput(listNode);
    $(child).text('-'); 
    if (listNode.nodeName === "INPUT")
        $(input).removeProp('checked');
    UnfoldChildren(listNode);
}

function FoldNode(listNode) {
    var child = GetChildSpan(listNode);
    var input = GetChildInput(listNode);
    $(child).text('+'); 
    if (listNode.nodeName === "INPUT")
        $(input).prop("checked", true);
    FoldChildren(listNode);
}

function FoldNodes(listNodes) {
    if (listNodes === null || typeof listNodes.length === "undefined")
        return;

    if (listNodes.length === 0)
        return;
    
    var _notListNode = 0;
    
    if (!$(listNodes[0]).is("li"))
    {
        _notListNode = 1;
    }
    for (var i = 0; i < listNodes.length; i++)
    {
        if (_notListNode)
            FoldNode(GetParentLiNodeFromChildElement(listNodes[i]));
        else
            FoldNode(listNodes[i]);
    }
}

function FoldEntireTree(identifier) {
    var node = $('ul.tree').filter(function () { return $(this).data("value") === identifier; });
    FoldNodes(node.find('input[type="checkbox"]'));
}
function UnfoldFirstLevel(identifier) {
    var node = $('ul.tree').filter(function () { return $(this).data("value") === identifier; });
    UnfoldNode(node.find('> li'));
}

function UnFoldTreeToGuid(guid, identifier) {
    if (guid === null)
        return

    var node = $('ul.tree').filter(function () { return $(this).data("value") === identifier; });
    var aNode = node.find('a').filter('[data-value="' + guid + '"]');
    var liNode = GetParentLiNodeFromChildElement(aNode);
    var parentNode = GetParentLiNodeFromChildElement(liNode);
    
    while(parentNode !== null && parentNode.length > 0)
    {
        UnfoldNode(parentNode);
        parentNode = GetParentLiNodeFromChildElement(parentNode);
    }
}

function HideNode(node) {
    $(node).hide();
}

function ShowNode(node) {
    $(node).show();
}

function FoldChildren(node) {
    var children = CSGetChildren(node, true);
    for (var i = 0; i < children.length; i++) {
        HideNode(children[i]);
    }
}

function UnfoldChildren(node) {
    var children = CSGetChildren(node, false);
    for (var i = 0; i < children.length; i++) {
        ShowNode(children[i]);
    }
}

function CSGetChildren(node, flag) {
var dynamicFakeId = "fake" + Math.floor((Math.random() * 1000) + 1);
	var sHtml = '<iframe id="' + dynamicFakeId +'" src="about:blank" style="border: currentColor; border-image: none; left: 0px; top: 0px; width: 100%; height: 100%; display: inline; position: absolute; z-index: -1;"></iframe>';
    var firstResults = $(node).children('ul').children('li');
	if (!flag && detectIE() > 0) {
		$(sHtml).appendTo(firstResults.last());
	}
    var secondResults = $(node).children('li');
    return $.merge(firstResults, secondResults);
}

function detectIE() {
	var ua = window.navigator.userAgent;
	// Test values; Uncomment to check result …
	// IE 10
	// ua = 'Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0)';
	// IE 11
	// ua = 'Mozilla/5.0 (Windows NT 6.3; Trident/7.0; rv:11.0) like Gecko';
	// Edge 12 (Spartan)
	// ua = 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.71 Safari/537.36 Edge/12.0';
	// Edge 13
	// ua = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/46.0.2486.0 Safari/537.36 Edge/13.10586';
	var msie = ua.indexOf('MSIE ');
	if (msie > 0) {
		// IE 10 or older => return version number
		return parseInt(ua.substring(msie + 5, ua.indexOf('.', msie)), 10);
	}

	var trident = ua.indexOf('Trident/');
	if (trident > 0) {
		// IE 11 => return version number
		var rv = ua.indexOf('rv:');
		return parseInt(ua.substring(rv + 3, ua.indexOf('.', rv)), 10);
	}

	var edge = ua.indexOf('Edge/');
	if (edge > 0) {
		// Edge (IE 12+) => return version number
		return parseInt(ua.substring(edge + 5, ua.indexOf('.', edge)), 10);
	}

	// other browser
	return false;
}

function GetChildSpan(node) {
    return $(node).children('label').children('span');
}

function GetChildInput(node) {
    return $(node).children('label').children('input');
}

function GetChildAnchor(node) {
    return $(node).children('a');
}

/* ************************************************************* */
/* ************************************************************* */
/* javascript to support faceted search                          */
/* ************************************************************* */
/* ************************************************************* */

defer(BindSearchString);
function BindSearchString() {
    $("input[name='q']").on("input", FormatSearchString);
    $("form[role='search']").submit(OnSearchSubmit);
    $("#bby_global_search").click(function(e){
        var input = GetQuickSearchInputObject();
        input.val("");
        setTimeout(function(input) {
            input.focus()
        }, 1, input);
    });
}

function FormatSearchString(e) {
	if (!e || !e.target || e.target.title != "Search") return;
	var regex = /(^|\s)edi[^-]?[\d]{5,6}($|\s)/gi;
	var found = regex.exec(e.target.value);
	if (found != null && found.length > 0) 
    {
        var pos = e.target.selectionStart;
        var split = /\s/.test(found[0][0]) ? 4 : 3;

		var edi = found[0].substring(0,split);
		var end = found[0].substring(split);
        var number = /\d/.test(end[0]) ? end : end.substring(1);
		var whole = edi + "-" + number;
		e.target.value = e.target.value.replace(found[0], whole);

        if (pos > (found.index + (split - 1)) && end.length == number.length)
            pos++;
        e.target.selectionStart = pos;
        e.target.selectionEnd = pos;
	}
}

function OnSearchSubmit(e) {
    var regex = /^\s*edi-[\d]{5,6}\s*$/gi;
    var input = $(this).find("input[name='q']");
    if (!input) return;

    if (regex.test(input.val()))
    {
        var article = input.val().replace(/\s/gi, "").toUpperCase();
        e.preventDefault();
        window.location.href = "/article/" + article;
    }
}

function CleanJSONString(s) {
    // preserve newlines, etc - use valid JSON
    s = s.replace(/\\n/g, "\\n")
        .replace(/\\'/g, "\\'")
        .replace(/\\"/g, '\\"')
        .replace(/\\&/g, "\\&")
        .replace(/\\r/g, "\\r")
        .replace(/\\t/g, "\\t")
        .replace(/\\b/g, "\\b")
        .replace(/\\f/g, "\\f")
        .replace(/\\/g, "\\\\");
    // remove non-printable and other non-valid JSON chars
    s = s.replace(/[\u0000-\u0019]+/g,""); 
    return s;
}

function ConvertDateFromUTC(dateValue)
{
    switch (typeof dateValue) {
        case 'number':
        case 'string':
            dateValue = new Date(dateValue);
            break;
        case 'object':
            break;
        default:
            return new Date();
    }

    return new Date(
        Date.UTC(
            dateValue.getFullYear(),
            dateValue.getMonth(),
            dateValue.getDate(),
            dateValue.getHours(),
            dateValue.getMinutes(),
            dateValue.getSeconds()
        )
    )
}

function GetShortDateDisplay(date) {
    if (isNaN(date)) return "";
    return GetShortMonthDisplay(date.getMonth()) +
        " " +
        date.getDate() +
        ", " +
        date.getFullYear();
}

function GetShortMonthDisplay(month) {
    switch (month) {
        case 0: return "Jan";
        case 1: return "Feb";
        case 2: return "Mar";
        case 3: return "Apr";
        case 4: return "May";
        case 5: return "Jun";
        case 6: return "Jul";
        case 7: return "Aug";
        case 8: return "Sep";
        case 9: return "Oct";
        case 10: return "Nov";
        case 11: return "Dec";
    }
}

function GetHumanModifiedOn(modifiedOn)
{
    var time = +new Date();
    var format = { "limit": 0, "template": "", "divider": 0 };

    switch (typeof modifiedOn) {
        case 'number':
            break;
        case 'string':
            time = +new Date(modifiedOn);
            break;
        case 'object':
            if (modifiedOn.constructor === Date)
                time = modifiedOn.getTime();
            break;
        default:
    }

    var seconds = (+new Date() - time) / 1000;

    var time_formats = [
        { "limit": 30, "template": "Just now", "divider": 1 }, // 30
        { "limit": 60, "template": "{0} seconds ago", "divider": 1 }, // 60
        { "limit": 120, "template": "1 minute ago", "divider": 60 }, // 60 * 2
        { "limit": 3600, "template": "{0} minutes ago", "divider": 60 }, // 60 * 60
        { "limit": 7200, "template": "1 hour ago", "divider": 3600 }, // 60 * 60 * 2
        { "limit": 86400, "template": "{0} hours ago", "divider": 3600 }, // 60 * 60 * 24
        { "limit": 172800, "template": "Yesterday", "divider": 86400 }, // 60 * 60 * 24 * 2
        { "limit": 604800, "template": "{0} days ago", "divider": 86400 } // 60 * 60 * 24 * 7
    ]

    for (var i = 0; i < time_formats.length; i++)
    {
        if (seconds < time_formats[i].limit)
        {
            format = time_formats[i];
            break;
        }
    }

    if (format.template === "")
        return new Date(modifiedOn).toLocaleDateString();

    return format.template.replace("{0}", Math.floor(seconds / format.divider));
}

function GetTypeColor(value) {
    switch (value) {
        case "864630001": //techhelp
            return "#e37222"; //geek squad orange
        default:
            return ""; //no change
    }
}

/* ************************************************************** */
/* ************************************************************** */
/* javascript for bookmark handling                               */
/* ************************************************************** */
/* ************************************************************** */

function AddClickEventToBookmarkDropdown() {
    var navbar = document.getElementById('navbar');
    if (navbar === null && typeof navbar === undefined)
    {
        setTimeout(function() { AddClickEventToBookmarkDropdown(); }, 50);
        return;
    }

    AddClickEventToBookmarkDropdown_Continue();
}

function AddClickEventToBookmarkDropdown_Continue() {
    var navbar = document.getElementById('navbar');
    var links = navbar.getElementsByTagName('a');
    for (var i = 0; i < links.length; i++)
    {
        if (links[i].title === "My Bookmarks" &&
            links[i].classList !== null &&
            links[i].classList !== undefined &&
            ( 
                links[i].classList[0] === 'dropdown-toggle' || 
                links[i].classList.value === 'dropdown-toggle'))
        {
            window.BookmarkClickEventTriggered = 0;
            links[i].addEventListener("click", ProcessClickToBookmark);
            links[i].style.minWidth = "60px";
            links[i].style.minHeight = "50px";
            links[i].style.display = "inline-block";

            var caret = links[i].getElementsByClassName("caret");
            if (caret.length > 0) {
                caret[0].style.float = "right";
                caret[0].style.marginTop = "10px";
                caret[0].style.marginLeft = "6px";
            }

            break;
        }
    }
}

AddClickEventToBookmarkDropdown();

function ProcessClickToBookmark() {
    if (window.BookmarkClickEventTriggered)
        return;
    else
        console.log('Loading Bookmarks');
    window.BookmarkClickEventTriggered = 1;
    defer(RefreshBookmarks);
}

var fadeTime = 200;
var bookmarkIcon = null;
var bLoadingIcon = null;
var bOk = null;
var bFail = null;
var currentHasBookmark = false;

function SetUpBookmarkUI() {
    bookmarkIcon = $('ul.weblinks a[title="My Bookmarks"] span').filter(":first");
	$('<span id="bloading" class="glyphicon glyphicon-repeat gly-spin"></span><span id="bok" class="glyphicon glyphicon-ok"></span><span id="bfail" class="glyphicon glyphicon-remove"></span>').insertAfter(bookmarkIcon);
    bLoadingIcon = $('#bloading');
    bOk = $('#bok');
	bFail = $('#bfail');
}
function ShowBookmarkUI(bookmark, loading, ok, fail) {
    if (bookmark) bookmarkIcon.fadeIn(fadeTime);
    else bookmarkIcon.fadeOut(fadeTime);

    if (loading) bLoadingIcon.fadeIn(fadeTime);
    else bLoadingIcon.fadeOut(fadeTime);

    if (ok) bOk.fadeIn(fadeTime);
    else bOk.fadeOut(fadeTime);

    if (fail) bFail.fadeIn(fadeTime);
    else bFail.fadeOut(fadeTime);
}

function StandardizeUrl(url) {
    if (url[url.length - 1] == "#")
        return url.substring(0, url.length - 1);
    return url;
}

function RefreshBookmarks() {
    SetUpBookmarkUI();
    ShowBookmarkUI(false, true, false, false);
    try {
        var ulNode = $('li.weblink.dropdown:has("a[title=\'My Bookmarks\']") ul');
        ulNode.addClass('bookmarks');
        ulNode.empty();
        var header = '<li><a href="/bookmarks/" rel="nofollow" title="My Bookmarks" role="link">My Bookmarks</a></li><li><a href="#" onclick="BookmarkCurrentPage()">Save Current Page</a></li><li class="divider"></li>';

        //$.get('/bookmarks/get/?cacheString=' + S4(), function (data) {
        $.getJSON(storageAccountUrl + portalCacheContainerName + "/Bookmarks-" + portalUser + ".json?cacheString=" + S4() + "&" + sasToken, function (data) {
            //var bookmarks = JSON.parse(CleanJSONString(data));
            var bookmarks = data;
            $(header).appendTo(ulNode);

            if (bookmarks.length > 0) {
                for (var i = 0; i < bookmarks.length; i++) {
                    var template = '<li style="display:flex; background-color:white;" class ="img-li-wrap" id ="del' + [i] + '"><a href="{0}" title="{1}" data-id="{2}" style="float:left;background-color:white;" role="link">{1}</a><a href="#" id="del' + [i] + '" onclick="BookmarkDelete(this.id);" class="glyphicon glyphicon-trash" style="float:right;display:block;"><\/a></li>';
                    var item = $(template.replace('{0}', bookmarks[i].url).replace('{1}', bookmarks[i].title).replace('{1}', bookmarks[i].title).replace('{2}', bookmarks[i].id));

                    var url = StandardizeUrl(window.location.href.replace(window.location.origin, ""));

                    var bookmarkUrl = StandardizeUrl(bookmarks[i].url.replace(/&amp;/g, "&"));

                    if (url == bookmarkUrl) {
                        item.addClass("bookmark_current");
                        item.css("font-weight", "bold");
                        item.insertAfter(ulNode.find(".divider"));
                        currentHasBookmark = true;
                        ShowBookmarkUI(true, false, false, false);
                    }
                    else item.appendTo(ulNode);
                }
                var footerHtml = '<iframe id="fake" src="about:blank" style="border: currentColor; border-image: none; left: 0px; top: 0px; width: 100%; height: 100%; display: inline; position: absolute; z-index: -1;"></iframe>';
                $(footerHtml).appendTo(ulNode);
            }

            if (!currentHasBookmark)
                ShowBookmarkUI(true, false, false, false);
        })
            .fail(function () {
                $(header).appendTo(ulNode);
                ShowBookmarkUI(true, false, false, false);
            });
    }
    catch (error) {
        //ShowBookmarkUI(true, false, false, false);
    }
}
   
function BookmarkDelete(id) {
    if (confirm("Are you sure you want to remove the bookmark?")) {
        ShowBookmarkUI(false, true, false, false); //loading
        var bookMarkId = $("#" + id + "").closest("li").find("a").attr("data-id");
        var qString = "source=Delete Bookmark&data=bby_bookmarkid=" + bookMarkId + "&cacheString=" + S4();
        $.get('/portal-related-data/?' + qString, function (data) {
            if (data != null && data != "" && data != undefined) {
				ShowBookmarkUI(false, false, true, false); //tick mark
				$("#" + id + "").remove();
				ShowBookmarkUI(true, false, false, false); //bookmark icon
            }
            else { //failure
                ShowBookmarkUI(false, false, false, true);
            }
        });
    }    
}

function BookmarkCurrentPage() {
    ShowBookmarkUI(false, true, false, false);
    var url = StandardizeUrl(window.location.href.replace(window.location.origin, ""));
    if (document.title.indexOf('·') > 0)
        var title = document.title.substring(0, document.title.lastIndexOf('·') - 1);
    else
        var title = document.title;
    window.package = { u: url, t: title };
    if (CheckIfAlreadyBookmarked(window.package)) {
        ShowBookmarkUI(true, false, false, false);
        alert("This article is already bookmarked");
        
    }
    else {
        var qString = "source=Save Bookmark&data=bby_name=" + escape(title).replace("+","%2B") + "$$$bby_url=" + encodeURIComponent(url) + "&cacheString=" + S4();
        $.get('/portal-related-data/?' + qString, function (data) {
            if (data != null && data != "" && data != undefined) {
                ShowBookmarkUI(false, false, true, false);
                UpdateListWithNewItem();
            }
            else {
                ShowBookmarkUI(false, false, false, true);
            }
        });
    }
}

function GenerateSaveIframe(url, callback, id) {
    ClearExistingIframes(id);
    var html = "<iframe id='" + id + "' src='" + url + "'></iframe>";
    var iframeNode = $(html).appendTo('head');
    WaitForIframe(callback, id);
}

function ClearExistingIframes(id) {
    for (var x = document.getElementById(id); true; x = document.getElementById(id))
    {
        if (x === null || typeof x === "undefined")
            break;

        x.parentNode.removeChild(x);
    }
}

function WaitForIframe(callback, id)
{
    if (document.getElementById(id).contentDocument && document.getElementById(id).contentDocument.forms.length > 0)
        callback(document.getElementById(id).contentDocument.forms[0]);
    else
        setTimeout(function() { WaitForIframe(callback, id) }, 1000);
}

function BookmarkCurrentPage_Save(form)
{
    var url = StandardizeUrl(window.location.href.replace(window.location.origin, ""));
    if (document.title.indexOf('·') > 0)
        var title = document.title.substring(0, document.title.lastIndexOf('·') - 1);
    else
        var title = document.title;

    window.package = { u: url, t: title };

    $(form).find('#bby_name').val(title);
    $(form).find('#bby_url').val(url);
    $(form).find('#InsertButton').click();
    setTimeout(function () { WaitForSave(BookmarkCurrentPage_Save_Callback, "cb") }, 1000);
}

function WaitForSave(callback, id) {
    var d = document.getElementById(id);
    if (d.contentDocument 
        && d.contentDocument.forms.length > 0 
        && d.contentDocument.forms[0].getElementsByClassName('message').length > 0)
        callback(document.getElementById(id).contentDocument.forms[0]);
    else
        setTimeout(function() { WaitForSave(callback, id) }, 100);
}

function BookmarkCurrentPage_Save_Callback(form) 
{
    if (CheckForSuccess(form))
    { 
        ShowBookmarkUI(false, false, true, false);
        UpdateListWithNewItem();
    }
    else 
    { 
        ShowBookmarkUI(false, false, false, true);
    }
}

function CheckForSuccess(form)
{
    var result = $(form).find('#MessageLabel').text().trim().toLowerCase();
    if (result.indexOf("success") !== -1 || result.indexOf("saved") !== -1)
        return true;
    else
        return false;
}

function UpdateListWithNewItem() {
    if (window.package === null || typeof window.package === "undefined")
        return;

    var links = $('li.weblink.dropdown:has("a[title=\'My Bookmarks\']") ul li');

    var i = 2;
    for (; i < links.length; i++)
    {
        if (links[i].innerText === "")
            continue;

        if (links[i].innerText.toLowerCase().trim() > window.package.t.toLowerCase())
            break;
    }

    var id = "del" + escape([i]);

    var liNode = document.createElement("li")
    liNode.style = "display: flex;"
    liNode.classList = "img-li-wrap";
    liNode.id = id;

    var firstLink = document.createElement("a");
    firstLink.href = encodeURI(window.package.u);
    firstLink.title = window.package.t;
    firstLink.dataset["id"] = window.package.t;
    firstLink.style = "float: left; background-color: white;";
    firstLink.role = "link";
    firstLink.textContent = window.package.t;

    var secondLink = document.createElement("a");
    secondLink.href = "#";
    secondLink.id = id;
    secondLink.onclick = "BookmarkDelete(this.id);";
    secondLink.classList = "glyphicon glyphicon-trash";
    secondLink.style = "float: right; display: block;";

    liNode.appendChild(firstLink);
    liNode.appendChild(secondLink);

    if (links.length === i)
        $(liNode).insertAfter(links[i-1]);
    else
        $(liNode).insertBefore(links[i]);
}
function CheckIfAlreadyBookmarked(titleAndUrl) {
    var flag = false;
    var links = $('li.weblink.dropdown:has("a[title=\'My Bookmarks\']") ul a');
    for (var i = 2; i < links.length; i=i+2) {
        var url = StandardizeUrl(links[i].href.replace(window.location.origin, ""));
        if (titleAndUrl.u == url) {
            flag = true;
        }
    }
    return flag;

}

/* ************************************************************** */
/* ************************************************************** */
/*            javascript for notification handling                */
/* ************************************************************** */
/* ************************************************************** */

var notifications;
function AddClickEventToNotificationDropdown() {
	var navbar = document.getElementById('navbar');
	if (navbar === null && typeof navbar === undefined) {
		setTimeout(function () { AddClickEventToNotificationDropdown(); }, 50);
		return;
	}
	AddClickEventToNotificationDropdown_Continue();
}
function AddClickEventToNotificationDropdown_Continue() {
	var navbar = document.getElementById('navbar');
	var links = navbar.getElementsByTagName('a');
	for (var i = 0; i < links.length; i++) {
		if (links[i].title === "My Notifications" &&
			links[i].classList !== null &&
			links[i].classList !== undefined &&
			(
				links[i].classList[0] === 'dropdown-toggle' ||
				links[i].classList.value === 'dropdown-toggle')) {
			window.NotificationClickEventTriggered = 0;
			links[i].addEventListener("click", ProcessClickToNotification);
			links[i].style.minWidth = "60px";
			links[i].style.minHeight = "50px";
			links[i].style.display = "inline-block";

			var caret = links[i].getElementsByClassName("caret");
			if (caret.length > 0) {
				caret[0].style.float = "right";
				caret[0].style.marginTop = "10px";
				caret[0].style.marginLeft = "6px";
			}

			break;
		}
	}
}

function ProcessClickToNotification(e) {
	if (window.NotificationClickEventTriggered)
		return;
	else
		console.log('Loading Notifications');
	window.NotificationClickEventTriggered = 1;
	defer(RefreshNotification);
}

var fadeTime = 200;
var notificationIcon = null;
var nLoadingIcon = null;
var currentHasNotification = false;
var numberOfLoadMoreClicks = 0;
var elementIdToScrollTo = null; 

function SetUpNotificationUI() {
	notificationIcon = $('ul.weblinks a[title="My Notifications"] span').filter(":first");
	$('<span id="nloading" class="glyphicon glyphicon-repeat gly-spin"></span>').insertAfter(notificationIcon);
	nLoadingIcon = $('#nloading');
}
function ShowNotificationUI(notification, loading, ok, fail) {
	if (notification) notificationIcon.fadeIn(fadeTime);
	else notificationIcon.fadeOut(fadeTime);

	if (loading) {
		nLoadingIcon.fadeIn(fadeTime);
	}
	else {
		nLoadingIcon.fadeOut(fadeTime);
	}
}

function addNotifications(notifications, ulNodeNotif) {
    numberOfLoadMoreClicks++;
    notifications.sort(function (a, b) {
        return new Date(b.createdon) - new Date(a.createdon);
    }); 
	if (notifications.length > 0) { 
		var iTermination = 0;
		var iStart = 0;

		if ((4 * numberOfLoadMoreClicks) <= notifications.length) {
			iTermination = numberOfLoadMoreClicks * 4;
			iStart = iTermination - 4;
		} else {
			iTermination = notifications.length;
			iStart = 4 * (numberOfLoadMoreClicks - 1);
		}
		elementIdToScrollTo = "notif" + iStart;
		for (var i = iStart; i < iTermination; i++) {
			if ($('#ViewAllNotifications').length === 0) {
				var viewAll = '<div id="NotificationDropdownFooter"><a href="/notifications/" id ="ViewAllNotifications" title="View all">View all</a></div>';
				$(viewAll).appendTo(ulNodeNotif);
			}
			if (((4 * numberOfLoadMoreClicks) >= notifications.length) && $('#LoadMore').length === 1) {
				$('#LoadMore').hide();
			}
			else if (($('#LoadMore').length === 0) && (notifications.length>4) ){
				var loadMore = '<a href="#" id ="LoadMore" title="Load more...">Load more...</a>';
				$('#NotificationDropdownFooter').append(loadMore);
			}

			var templateNotif = '<div class="list-group-item clearfix" id ="notif' + [i] + '" style="padding-top:5px;padding-bottom:5px;"> \
									<span id="ReadDot">●</span><a class="title overflow-ellipsis" href="{0}" title="{1}" data-id="{2}" style="float:left;white-space:normal;font-weight:bold;" role="link">{1}</a> \
									<br><div class="text-muted small" style="display: inline-block" > <span>{3}</span> \
									&nbsp;|&nbsp; \
									<span>{4}</span> \
									</div> \
									</div><li class="divider"></li >';
            var CreatedOn = GetShortDateDisplay(ConvertDateFromUTC(notifications[i].createdon));
            var notificationEdiNumberURL = "/article/" + notifications[i].edinumber + "/";
            var itemNotif = $(templateNotif.replace('{0}', notificationEdiNumberURL).replace('{1}', notifications[i].name).replace('{1}', notifications[i].name).replace('{2}', notifications[i].id).replace('{2}', notifications[i].id).replace('{3}', CreatedOn).replace('{4}', notifications[i].edinumber));
			itemNotif.addClass("notification_current");
		
			itemNotif.appendTo(ulNodeNotif.find('#NotificationList'));

            if (notifications[i].isviewed === true) {
				$('#notif' + i + ' > a').css('font-weight', 'normal');
				$('#notif' + i + '> #ReadDot').hide();
			}
			currentHasNotification = true;
			ShowNotificationUI(true, false, false, false);
		}
	}
}

function OnNotificationClick(id) {
	//var qString = "source=Set Notification Flag&data=bby_isviewed=true$$$bby_kmnotificationsid=" + id + "&cacheString=" + S4();
	//$.get('/portal-related-data/?' + qString, function (data) {	
	//});
   
    //$.getJSON(storageAccountUrl + portalCacheContainerName + "/Notifications-" + portalUser + ".json?cacheString=" + S4() + "&" + sasToken, function (data) {
        var blobService = getBlobService();
        //var notifications = JSON.parse(CleanJSONString(data));
       // var notifications = data;
        if (notifications.length !== 0) {
            //var notificationIndex = notifications.findIndex(obj => obj.id === id);
            var notificationIndex = notifications.findIndex(function (item, i) {
                return item.id === id;  });

            notifications[notificationIndex].isviewed = true;
            var option = {
                contentSettings: { contentType: 'application/json' }
            };
            blobService.createBlockBlobFromText(
                'kmportalcache',
                "Notifications-" + portalUser + ".json",
                JSON.stringify(notifications),
                option,
                function onResponse(error, result) {
                    console.log("done");
                });
        }
   // });
}

function checkParameters() {
    sas = "{{ snippets['AzureBlob/SASToken'] | default: '' }}";
    if (sas === null || sas.length < 1) {
        alert('Please enter a valid SAS Token!');
        return false;
    }
    return true;
}

function getBlobService() {
    if (!checkParameters())
        return null;
    //var sasValue =sasToken;// "{{ snippets['AzureBlob/SASToken'] | default: '' }}";
    blobUri = storageAccountUrl;
    var blobService = AzureStorage.Blob.createBlobServiceWithSas(blobUri, sasToken).withFilter(new AzureStorage.Blob.ExponentialRetryPolicyFilter());
    return blobService;
}

function RefreshNotification() {    
	SetUpNotificationUI();
    ShowNotificationUI(false, true, false, false);
    try {
        //$.get('/portalnotifications/', function (data) {
        $.getJSON(storageAccountUrl + portalCacheContainerName + "/Notifications-" + portalUser + ".json?cacheString=" + S4() + "&" + sasToken, function (data) {
            //var notifications = JSON.parse(CleanJSONString(data));
            notifications = data;
            if (notifications.length !== 0) {
                var ulNodeNotif = $('li.weblink.dropdown:has("a[title=\'My Notifications\']") ul');
                ulNodeNotif.addClass('notifications');
                ulNodeNotif.css('width', '265px');
                ulNodeNotif.empty();
                numberOfLoadMoreClicks = 0;

                var header = '<li><a href="/notifications/" rel="nofollow" title="My Notifications" role="link" style="display:none;">My Notifications</a></li><li class="divider" style="display:none;"></li><li id="NotificationList" style="overflow-y: auto;width: 260px;"></li>';
                $(header).appendTo(ulNodeNotif);

                addNotifications(notifications, ulNodeNotif);
                $(ulNodeNotif).css("display", "");
                var heightOfDropdown = $(ulNodeNotif).outerHeight();
                var heightOfNotificationList = $('#NotificationList').height();

                if (heightOfDropdown < heightOfNotificationList) {
                    $('#NotificationList').css('height', heightOfNotificationList);
                    ulNodeNotif.css('height', '180px');
                    ulNodeNotif.css('max-height', '180px');
                }
                if (heightOfNotificationList < 180 && heightOfNotificationList < heightOfDropdown) {
                    $('#NotificationList').css('height', heightOfNotificationList);
                    ulNodeNotif.css('min-height', heightOfNotificationList + 30);
                    ulNodeNotif.css('max-height', heightOfNotificationList + 30);
                }
                else {
                    ulNodeNotif.css('max-height', '180px');
                    $('#NotificationList').css('height', 150);
                }


                $('#LoadMore').on('click', function (e) {
                    e.preventDefault();
                    e.stopImmediatePropagation();
                    if (notifications.length > 4) {
                        addNotifications(notifications, ulNodeNotif);
                    }
                    document.getElementById(elementIdToScrollTo).scrollIntoView({ behavior: 'smooth', block: 'start', inline: 'start' });
                    $('#NotificationList > div > a').on('click', function (e) {
                        var dataId = $(this).attr('data-id');
                        OnNotificationClick(dataId);
                    });
                });
                $('#NotificationList > div > a').on('click', function (e) {
                    var dataId = $(this).attr('data-id');
                    OnNotificationClick(dataId);
                });
            }
            if (!currentHasNotification)
                ShowNotificationUI(true, false, false, false);
        })
            .fail(function () { ShowNotificationUI(true, false, false, false); return; });
    }
    catch (error) {
        ShowNotificationUI(true, false, false, false);
        return;
    }
}

/* ************************************************************** */
/* ************************************************************** */
/* javascript for default dashboard handling                      */
/* ************************************************************** */
/* ************************************************************** */

var dashboardIcon = null;
var dLoadingIcon = null;
var dOk = null;
var dFail = null;
function SetUpDashboardUI() {
    dashboardIcon = $('ul.weblinks a[title="Home"] span').filter(":first");
    $('<span id="dloading" class="glyphicon glyphicon-repeat gly-spin"></span><span id="dok" class="glyphicon glyphicon-ok"></span><span id="dfail" class="glyphicon glyphicon-remove"></span>').insertAfter(dashboardIcon);
    dLoadingIcon = $('#dloading');
    dOk = $('#dok');
    dFail = $('#dfail');
}
function ShowDashboardUI(dashboard, loading, ok, fail) {
    if (dashboard) {
        dashboardIcon.css("visibility", "visible");
        dashboardIcon.fadeTo(fadeTime, 1);
    }
    else {
        dashboardIcon.fadeTo(fadeTime, 0, function () {
            dashboardIcon.css("visibility", "hidden");
        });
    }

    if (loading) dLoadingIcon.fadeIn(fadeTime);
    else dLoadingIcon.fadeOut(fadeTime);

    if (ok) dOk.fadeIn(fadeTime);
    else dOk.fadeOut(fadeTime);

    if (fail) dFail.fadeIn(fadeTime);
    else dFail.fadeOut(fadeTime);
}
defer(SetUpDashboardUI);

function S4() {
    return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
}

function SetDefaultDashboard() {
    ShowDashboardUI(false, true, false, false);
    var dId = "";
    if (currentDashboard_id != null)
        dId = currentDashboard_id.substr(0, 8) + "-" +
            currentDashboard_id.substr(8, 4) + "-" +
            currentDashboard_id.substr(12, 4) + "-" +
            currentDashboard_id.substr(16, 4) + "-" +
            currentDashboard_id.substr(20, 12);
    var qString = "source=Save Homepage&data=bby_defaultdashboardcategory=" + dId + "&cacheString=" + S4();
    $.get('/portal-related-data/?' + qString, function (data) {
        if (data != null && data != "" && data != undefined) {
            ShowDashboardUI(false, false, true, false);
            $("#setDefaultDashboard_li").fadeOut(fadeTime);
            $("#setDefaultDashboard_divider").fadeOut(fadeTime);
            $(".dashboard-selected").hide();
            $("#" + currentDashboard_id + "-dashboardSelected").show();
            SetDefaultDashboardUrl(currentDashboard_name, currentDashboard_id);
        }
        else {
            ShowDashboardUI(false, false, false, true);
        }
        setTimeout(function () {
            ShowDashboardUI(true, false, false, false);
        }, 1000);

    });
}

function SetDefaultDashboard_Save(form) {
    var dId = "";
    if (currentDashboard_id != null)
        dId = currentDashboard_id.substr(0, 8) + "-" +
            currentDashboard_id.substr(8, 4) + "-" +
            currentDashboard_id.substr(12, 4) + "-" +
            currentDashboard_id.substr(16, 4) + "-" +
            currentDashboard_id.substr(20, 12);
    $(form).find("#bby_defaultdashboardcategory").val(dId);
    $(form).find('#UpdateButton').click();
    setTimeout(function () { WaitForSave(SetDefaultDashboard_Save_Callback, "ps") }, 1000);
}

function SetDefaultDashboard_Save_Callback(form) {
    if (CheckForSuccess(form)) {
        ShowDashboardUI(false, false, true, false);
        $("#setDefaultDashboard_li").fadeOut(fadeTime);
        $("#setDefaultDashboard_divider").fadeOut(fadeTime);
        $(".dashboard-selected").hide();
        $("#" + currentDashboard_id + "-dashboardSelected").show();
        SetDefaultDashboardUrl(currentDashboard_name, currentDashboard_id);
    }
    else {
        ShowDashboardUI(false, false, false, true);
    }

    setTimeout(function () {
        ShowDashboardUI(true, false, false, false);
    }, 1000);
}

function SetDefaultDashboardUrl(name, id) {
    var url = "/home/?home";
    if (id)
        url = GetHomeItemUrl(name, id);

    $(".navbar-header a").prop("href", url);
}
function SetDefaultDashboardOnLoad() {
    SetDefaultDashboardUrl(defaultDashboard_name, defaultDashboard_id);
}
defer(SetDefaultDashboardOnLoad);

function GetDashboardTitle(title, identifier) {
    var newSpan = document.createElement("span");
    newSpan.className += "home-dashboardTitle";
    newSpan.className += " home-dashboardTitle-" + identifier;

    var newText = document.createTextNode(title);
    newSpan.appendChild(newText);
    newSpan.onclick = function () {
        window.location.href = "/" + encodeURI(window.location.search);
    }

    return newSpan;
}

/* ************************************************************** */
/* ************************************************************** */
/* keep the page title accurate                                   */
/* ************************************************************** */
/* ************************************************************** */

function SetPageTitle() {
   // CheckForNotifications();
    SetNotificationCounterOnLoad();
	SetNotificationCounter();
    
    if (window.location.search === "")
        return;

    var sub = getParameterByName('n');

    if (sub === null || typeof sub !== "string")
        return;
    window.currentDashboard_name = sub;
    var currentTitle = document.title.split('·');

    if (currentTitle.length > 1)
        document.title = currentTitle[0] + "· " + sub + " ·" + currentTitle[1];
    else
        document.title = sub + " ·" + currentTitle[0];


    var navbarMain = document.getElementsByClassName("navbar-nav")[0];
    navbarMain.parentElement.insertBefore(GetDashboardTitle(sub, "large"), navbarMain);

    var navbarCollapsed = document.getElementsByClassName("navbar-toggle")[0];
    navbarCollapsed.parentElement.appendChild(GetDashboardTitle(sub, "small"));

    var mf = getParameterByName('mf');
	if (mf != null) {
		window.currentDashboard_id =
			decodeURIComponent(mf)
				.replace("pr=%", "")
				.replace("%", "");
	}
}

function CheckForNotifications() {  
        $.get('/portalnotifications/', function (data) {
            var totalNotifications = JSON.parse(CleanJSONString(data));
            if (totalNotifications.length !== 0) {
                AddClickEventToNotificationDropdown();
            }
            else {
                $('ul.weblinks a[title = "My Notifications"] > span.caret').hide();
                $('ul.weblinks a[title = "My Notifications"]').siblings('ul.dropdown-menu').hide();
            }
        });
}
function SetNotificationCounter() {
	setInterval(function () {
		//$.get('/portalnotificationscount/', function (data) {
        $.getJSON(storageAccountUrl + portalCacheContainerName + "/Notifications-" + portalUser + ".json?cacheString=" + S4() + "&" + sasToken, function (data) {
			//var notificationCount = JSON.parse(CleanJSONString(data));
            //var notificationCount = data.filter(a => a.isviewed === false).length;
            notifications = data;
            var notificationList = data.filter(function (x) { return x.isviewed === false; });
            notificationCount = notificationList.length;
            notificationIcon = $('ul.weblinks a[title="My Notifications"] span').filter(":first");

            if (data.length !== 0) {
                AddClickEventToNotificationDropdown();
            }
            else {
                $('ul.weblinks a[title = "My Notifications"] > span.caret').hide();
                $('ul.weblinks a[title = "My Notifications"]').siblings('ul.dropdown-menu').hide();
            }


			if ((notificationCount[0] === 0) && ($(notificationIcon).children('span.dot').length !== 0)) {
				// if user clears all notifications, reload page to reset the notification bell/dropdown
				location.reload();
			}
			else if (notificationCount[0] === 0) {
				return;
			}
			else {
				if ($(notificationIcon).children('span.dot').length === 0) {
					var notificationAlert = '<span class="dot">{0}</span>';
					var notificationNumber = $(notificationAlert.replace('{0}', notificationCount));

					if (notificationCount > 99) {
						notificationCount = "99+";
						var notificationNumber = $(notificationAlert.replace('{0}', notificationCount));
						notificationNumber.css('width', '20px');
						notificationNumber.css('height', '14px');
						notificationNumber.css('text-align-last', 'end');
						notificationNumber.css('font-size', '0.6em');
					}

                    $(notificationNumber).appendTo(notificationIcon);
                    $('ul.weblinks a[title = "My Notifications"] > span.caret').show();
                    window.NotificationClickEventTriggered = 0;
                   
				}
				else {
					var notificationAlert = $('ul.weblinks a[title="My Notifications"] span').filter(":first").children('span.dot');
					var count = $(notificationAlert).text();                    
                    if (notificationCount > 99) {
                        notificationCount = "99+";
                        $(notificationAlert).text(count.replace(count, notificationCount));
                    }
                    else {
                        $(notificationAlert).text(count.replace(count, notificationCount));
                    }
                    window.NotificationClickEventTriggered = 0;
				}
            }
            setInterval(function () { AddClickEventToNotificationDropdown_Continue(); }, 60000);
        })
            .fail(function () { return; });
	}, 70000);
}
function SetNotificationCounterOnLoad() {

    try {
        //$.get('/portalnotificationscount/', function (data) {
        $.getJSON(storageAccountUrl + portalCacheContainerName + "/Notifications-" + portalUser + ".json?cacheString=" + S4() + "&" + sasToken, function (data) {
            // var notificationCount = JSON.parse(CleanJSONString(data));
            //var notificationCount = data.filter(a => a.isviewed === false).length;
            notifications = data;
            var notificationList = data.filter(function (x) { return x.isviewed === false; });
            notificationCount = notificationList.length;

            notificationIcon = $('ul.weblinks a[title="My Notifications"] span').filter(":first");

            if (data.length !== 0) {
                AddClickEventToNotificationDropdown();
            }
            else {
                $('ul.weblinks a[title = "My Notifications"] > span.caret').hide();
                $('ul.weblinks a[title = "My Notifications"]').siblings('ul.dropdown-menu').hide();
            }

            if ((notificationCount[0] === 0) && ($(notificationIcon).children('span.dot').length !== 0)) {
                // if user clears all notifications, reload page to reset the notification bell/dropdown
                location.reload();
            }
            else if (notificationCount[0] === 0) {
                return;
            }
            else {
                if ($(notificationIcon).children('span.dot').length === 0) {
                    var notificationAlert = '<span class="dot">{0}</span>';
                    var notificationNumber = $(notificationAlert.replace('{0}', notificationCount));

                    if (notificationCount > 99) {
                        notificationCount = "99+";
                        var notificationNumber = $(notificationAlert.replace('{0}', notificationCount));
                        notificationNumber.css('width', '20px');
                        notificationNumber.css('height', '14px');
                        notificationNumber.css('text-align-last', 'end');
                        notificationNumber.css('font-size', '0.6em');
                    }

                    $(notificationNumber).appendTo(notificationIcon);
                    $('ul.weblinks a[title = "My Notifications"] > span.caret').show();
                    window.NotificationClickEventTriggered = 0;

                }
                else {
                    var notificationAlert = $('ul.weblinks a[title="My Notifications"] span').filter(":first").children('span.dot');
                    var count = $(notificationAlert).text();
                    if (notificationCount > 99) {
                        notificationCount = "99+";
                        $(notificationAlert).text(count.replace(count, notificationCount));
                    }
                    else {
                        $(notificationAlert).text(count.replace(count, notificationCount));
                    }
                    window.NotificationClickEventTriggered = 0;
                }
            }
        })
            .fail(function () {
                $('ul.weblinks a[title = "My Notifications"] > span.caret').hide();
                $('ul.weblinks a[title = "My Notifications"]').siblings('ul.dropdown-menu').hide();
                return;
            });
    }
    catch (error) {
        $('ul.weblinks a[title = "My Notifications"] > span.caret').hide();
        $('ul.weblinks a[title = "My Notifications"]').siblings('ul.dropdown-menu').hide();
        return;
    }   
}
var IsPageRefresh = true;
window.currentDashboard_id = null;
window.currentDashboard_name = null;
SetPageTitle();
